

proc Set_S9418_Options {} {
    global Options S9418_Base

    load_Options

   if {![info exists Options(s9418.channels)]} {set Options(s9418.channels) 4096}
   if {![info exists Options(s9418.offset)]} {set Options(s9418.offset) 0}
   if {![info exists Options(s9418.shift)]} {set Options(s9418.shift) 0}
   if {![info exists Options(s9418_GroupBase)]} {set Options(s9418_GroupBase) $S9418_Base}
   set S9418_Base  $Options(s9418_GroupBase)

    update_Options
}

proc Dump_Raw_VME_Data_S9418 {} {

   set info ""

   set maxmodule [inform vmecontrol inquire s9418count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   for {set i 1} {$i <= $maxmodule} {incr i} {
      append info "\nS9418#$i\n"
      execute-command 19 "Getting Raw Data for S9418#$i" "2 $i" 2
      set data [read-register EXEC.Response]

     foreach word $data {
         set word 0x$word
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 30 & 3]
         switch $type {
           0 {
             set event_type "data"
             set channel [expr $word >> 20 & 0x1f]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i;  adc= 0x%3x" $word $event_type $channel $adc]"
           }
           1 {
             set event_type "footer"
             set VSN [expr $word & 0xffff]
             set WC [expr $word >> 16 & 0x3f]
             set EvCount [expr $word >> 25 & 0xf]
             append info "[format "0x%8x    %8s   vsn= 0x%4x;   wc= %3i; Ev Counter= %3i" $word $event_type $VSN $WC $EvCount]"
           }
           2 -
           3 {
             set event_type "end event"
             append info "[format "0x%8x    %8s" $word $event_type]"
           }
         }
         append info "\n"
     }
   }
   return $info
}

proc define_S9418_config {} {
    global VMEserver
    global S9418_Base

   set maxmodule [inform vmecontrol inquire s9418count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "9418 0 [format "0x%x" $S9418_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/s9418#$i.Disable'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register s9418#$i.Disable"
          clear-footer
          error $m
      }
      DefineConfig "9418 $i [string range [first $m] 2 end]"
   }
}

proc create_S9418_spectra {} {
    global SpectrumServer S9418_Base
    global Options

   set maxmodule [inform vmecontrol inquire s9418count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 32

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module s9418#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName s9418#$i.adc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(s9418.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum s9418#$i.adc$j"
          clear-footer
          error $m
      } else {
          set group [expr $S9418_Base - 1 + $i]
          map_spectrum  "$spectrumname $group $j $Options(s9418.offset) $Options(s9418.shift)"
      }
    }
   }
}

